package com.unis.hlht.cjlksj.server.common.taskjobs;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.CronTask;
import org.springframework.scheduling.config.ScheduledTask;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import com.unis.hlht.cjlksj.server.common.rest.LiangKuRest;
import com.unis.hlht.cjlksj.server.dto.instruction.InstructionParasDTO;
import com.unis.hlht.cjlksj.server.entity.HKSBXX;
import com.unis.hlht.cjlksj.server.entity.HKZZJG;
import com.unis.hlht.cjlksj.server.entity.LiangKuApiEntity;
import com.unis.hlht.cjlksj.server.entity.TableCommentEntity;
import com.unis.hlht.cjlksj.server.enums.instruction.InstructionTypeEnum;
import com.unis.hlht.cjlksj.server.mapper.GrabResultMapper;
import com.unis.hlht.cjlksj.server.mapper.HKSBXXMapper;
import com.unis.hlht.cjlksj.server.mapper.HKZZJGMapper;
import com.unis.hlht.cjlksj.server.mapper.LiangKuApiMapper;
import com.unis.hlht.cjlksj.server.mapper.TableCommentMapper;
import com.unis.hlht.cjlksj.server.utils.DateUtil;
import com.unis.hlht.cjlksj.server.utils.ExcelUtil;
import com.unis.hlht.cjlksj.server.utils.OrderID;
import com.unis.hlht.cjlksj.server.utils.RootResult;

import lombok.Data;
import lombok.extern.log4j.Log4j2;

/**
 * 定时任务管理配置
 */
@Configuration
@EnableScheduling
//@Slf4j
@Log4j2
public class CompleteScheduleConfig implements SchedulingConfigurer {
    @Autowired
    private LiangKuRest liangKuRest;
	
    @Autowired
    private LiangKuApiMapper liangKuApiMapper;
    
    @Autowired
    private ExecutorService executorService;
	
	@Value("${deploy_system}")
	private String deploySystem;
	
    
    
	private static List<TaskRecord> taskRecordList = new ArrayList<>();
	
	
	public void addDataGrab() {
		//海康机构/库区信息同步
		TaskRecord taskRecord2 = new TaskRecord();
		taskRecord2.setExecuteMehod("zzjgPush");
		taskRecord2.setCron("0 5 21 ? * *");
		taskRecordList.add(taskRecord2);

		//海康设备信息同步
		TaskRecord taskRecord = new TaskRecord();
		taskRecord.setExecuteMehod("sbxxPush");
		taskRecord.setCron("0 10 21 ? * *");
		taskRecordList.add(taskRecord);
		
		//粮库信息抓取
		TaskRecord taskRecord3 = new TaskRecord();
		taskRecord3.setExecuteMehod("grabAllScheduled");
		// 每天中午12点10分执行
//		taskRecord3.setCron("0 10 23 ? * *");
		taskRecord3.setCron("0 0/60 * * * ?");
		taskRecordList.add(taskRecord3);

		//心跳检测
		TaskRecord taskRecord4 = new TaskRecord();
		taskRecord4.setExecuteMehod("excuteHeartCheck");
		taskRecord4.setCron("0 0/10 * * * ?");
		taskRecordList.add(taskRecord4);
	}
	
	public void addHeartCheck() {
		//心跳检测
		TaskRecord taskRecord4 = new TaskRecord();
		taskRecord4.setExecuteMehod("excuteHeartCheck");
		taskRecord4.setCron("0 0/10 * * * ?");
		taskRecordList.add(taskRecord4);
		
		//心跳检测结果，定时导出
		TaskRecord taskRecord5 = new TaskRecord();
		taskRecord5.setExecuteMehod("exportHeartCheckList");
		taskRecord5.setCron("0 0 0/1 * * ?");
		taskRecordList.add(taskRecord5);
	}

	
	

	
	@Override
	public void configureTasks(ScheduledTaskRegistrar taskRegistrar) {
		
		if(deploySystem.equals("xtjc")) {
			addHeartCheck();
		}else if(deploySystem.equals("cjxt")){
			addDataGrab();
		}else {
			return;
		}
		
		for (TaskRecord taskRecord : taskRecordList) {
			String cron = taskRecord.getCron();
			String executeMehod2 = taskRecord.getExecuteMehod();
			Runnable runnable = () -> {
				try {
					log.info("执行定时任务方法：",executeMehod2);
					if(executeMehod2.equals("excuteHeartCheck")) {
						excuteHeartCheck();
					}
					if(executeMehod2.equals("exportHeartCheckList")) {
						exportHeartCheckList();
					}
					if(executeMehod2.equals("zzjgPush")) {
						zzjgPush();
					}
					if(executeMehod2.equals("sbxxPush")) {
						sbxxPush();
					}
					if(executeMehod2.equals("grabAllScheduled")) {
						grabAllScheduled();
					}
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			};
			CronTask cronTask = new CronTask(runnable, cron);
			ScheduledTask scheduledTask = taskRegistrar.scheduleCronTask(cronTask);
			// scheduledTask.cancel(); 取消定时任务

		}

	}

	
    
	
    
	@Data
	private static class TaskRecord {

		private String classPath;

		private String executeMehod;

		private String cron;

		// 可以在增加一个type 执行其他类型的定时任务
	}
	
	
	
	//**心跳*******************************************************************************//
	public void exportHeartCheckList() {
		Map<String,Object> m = new HashMap<>();
		List<LiangKuApiEntity> mlist = liangKuApiMapper.findAll2(null);
		int count_success = 0;
		int sheng_count_success = 0;
		int shixian_count_success = 0;
		int xian_count_success = 0;
		for (LiangKuApiEntity liangKuApiEntity : mlist) {
			if(liangKuApiEntity.getIS_ONLINE().equals("1")) {
				liangKuApiEntity.setIS_ONLINE("在线");
				count_success = count_success + 1;
				if(liangKuApiEntity.getBELONGTO().equals("粮农-省级")) { sheng_count_success = sheng_count_success + 1; }
				if(liangKuApiEntity.getBELONGTO().equals("粮农-市县")) { shixian_count_success = shixian_count_success + 1; }
				if(liangKuApiEntity.getBELONGTO().equals("西安市")) { xian_count_success = xian_count_success + 1; }
				
			}else {
				liangKuApiEntity.setIS_ONLINE("不在线");
			}
		}
		
		m.put("sheng_count_success", sheng_count_success);
		m.put("shixian_count_success", shixian_count_success);
		m.put("xian_count_success", xian_count_success);
		int count = mlist.size();
		m.put("count",count);
		m.put("count_success", count_success);
		String now = DateFormat.getDateTimeInstance().format(new Date());
		m.put("now", now);
		m.put("list", mlist);
		
		LiangKuApiEntity total = new LiangKuApiEntity();
		total.setBELONGTO(
				"Hi现在时间是 " + now + "____总库点：" + count + "____在线库点：" + count_success + " [{省级：" + sheng_count_success
						+ "}" + "{市县：" + shixian_count_success+"}"+"{西安："+xian_count_success+"}]");
		mlist.add(total);
        //excel标题
        String[] title = {"所属", "库点名称", "是否在线", "检测时间", "检测地址"};
        //sheet名
        String sheetName = "粮库上传明细";
        String[][] content = new String[mlist.size()][5];
        for (int i = 0; i < mlist.size(); i++) {
            LiangKuApiEntity obj = mlist.get(i);
            content[i][0] = obj.getBELONGTO();
            content[i][1] = obj.getName();
            content[i][2] = obj.getIS_ONLINE();
            content[i][3] = obj.getCHECK_TIME();
            content[i][4] = obj.getUrl();
        }
        //创建HSSFWorkbook
        HSSFWorkbook wb = ExcelUtil.getHSSFWorkbook(sheetName, title, content, null);
        //响应到客户端
        try {
        	String path = "D://XIN_TIAO_JIAN_CE";
        	String currentDateTime16Str = DateUtil.getCurrentStringTimestamp();
			File file = new File(path+"//"+currentDateTime16Str + ".xls");
        	OutputStream outputStream = new FileOutputStream(file);
            wb.write(outputStream);
            outputStream.flush();
            outputStream.close();
        } catch (Exception e) {
        	e.printStackTrace();
        }
	}
	
    /**
     * 心跳检查
     */
	public void excuteHeartCheck() {
		// 获取所有粮库的外网地址
		List<LiangKuApiEntity> list = liangKuApiMapper.findAll2(null);
		for (LiangKuApiEntity api : list) {
			executorService.execute(new Runnable() {
				@Override
				public void run() {
					String name = api.getName();
					String result = "";
					String url = api.getUrl();
					// TODO Auto-generated method stub
					boolean heart = false;
					try {
						heart = connectCheck(url,api.getDwdm());
					} catch (Exception e) {
						result = name + ":心跳检查_fail_exception_____" + " 接口:" + url;
						heart = false;
					}
					if (heart) {
						result = name + ":心跳检查_success__" + " 接口:" + url;
						liangKuApiMapper.updateIsOnlineById("1", DateUtil.getCurrentDateStr(), api.getId(), "T");
					} else {
						result = name + ":心跳检查_fail_____" + " 接口:" + url;
						liangKuApiMapper.updateIsOnlineById("0", DateUtil.getCurrentDateStr(), api.getId(),"F");
					}
					log.info(result);
				}
			});
		}
    }

    private boolean connectCheck(String url,String dwdm) throws InterruptedException {
    	//心跳指令请求体数据
    	try {
    		InstructionParasDTO dto = new InstructionParasDTO();
    		dto.setOrderId(UUID.randomUUID().toString());
    		dto.setType(InstructionTypeEnum.hearBeatInstruction.getCode().toString());
    		dto.setDwdm(dwdm);
    		//一次尝试
			if (liangKuRest.hearIssure(url, dto)) {
				return true;
			}
    		return false;
    	} catch (Exception e) {
    		log.error(e);
    		return false;
    	}
    }
	//*******************************心跳*******************************//

	
	//*视频摄像头*//
	@Autowired
	private HKZZJGMapper hkzzjgMapper;

	@Autowired
	private HKSBXXMapper hksbxxMapper;

	public String GetCameraPreviewURL(Integer pageNo, Integer pageSize, Integer type) {
		/**
		 * STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.
		 */
		// ArtemisConfig.host = "106.3.145.75:443"; // artemis网关服务器ip端口
		// ArtemisConfig.appKey = "28116466"; // 秘钥appkey 28116466
		// ArtemisConfig.appSecret = "W9zihM0hNRsnNic3zuKM";// 秘钥appSecret
		// W9zihM0hNRsnNic3zuKM

		//陕西
		ArtemisConfig.host = "192.168.11.101:443"; // artemis网关服务器ip端口
		ArtemisConfig.appKey = "21625125"; // 秘钥appkey
		ArtemisConfig.appSecret = "6ATtM86JibfyDrWkdgn9";// 秘钥appSecret

		// 政务外网
		/*
		 * ArtemisConfig.host = "192.168.7.101:443"; // artemis网关服务器ip端口
		 * ArtemisConfig.appKey = "22129988"; // 秘钥appkey 28116466
		 * ArtemisConfig.appSecret = "vHvyP8OJV5uLoApXMaGS";// 秘钥appSecret
		 * W9zihM0hNRsnNic3zuKM
		 */
		/**
		 * STEP2：设置OpenAPI接口的上下文
		 */
		final String ARTEMIS_PATH = "/artemis";

		String body = null;

		String contentType = null;

		Map path = new HashMap();

		if (type == 1) { // 获取区域列表
			/**
			 * STEP3：设置接口的URI地址
			 */
			final String previewURLsApi = ARTEMIS_PATH + "/api/irds/v2/region/nodesByParams"; // 获取区域列表
			path = new HashMap(2) {
				{
					put("https://", previewURLsApi);// 根据现场环境部署确认是http还是https
				}
			};

			/**
			 * STEP4：设置参数提交方式
			 */
			contentType = "application/json";

			/**
			 * STEP5：组装请求参数
			 */
			JSONObject jsonBody = new JSONObject();

			// 获取区域列表所要传的参数
			jsonBody.put("pageNo", pageNo);
			jsonBody.put("pageSize", pageSize);
			jsonBody.put("resourceType", "camera");
			jsonBody.put("cascadeFlag", 0);

			body = jsonBody.toJSONString();
		} else if (type == 2) {

			/**
			 * STEP3：设置接口的URI地址
			 */
			final String previewURLsApi = ARTEMIS_PATH + "/api/resource/v1/cameras"; // 获取分页监控点资源
			path = new HashMap(2) {
				{
					put("https://", previewURLsApi);// 根据现场环境部署确认是http还是https
				}
			};

			/**
			 * STEP4：设置参数提交方式
			 */
			contentType = "application/json";

			/**
			 * STEP5：组装请求参数
			 */
			JSONObject jsonBody = new JSONObject();

			// 获取分页监控点资源参数
			jsonBody.put("pageNo", pageNo);
			jsonBody.put("pageSize", pageSize);

			body = jsonBody.toJSONString();
		} else {

			/**
			 * STEP3：设置接口的URI地址
			 */
			// final String previewURLsApi = ARTEMIS_PATH + "/api/nms/v1/online/camera/get";
			// // 获取分页监控点资源
			// path = new HashMap(2) {
			// {
			// put("https://", previewURLsApi);// 根据现场环境部署确认是http还是https
			// }
			// };
			//
			// /**
			// * STEP4：设置参数提交方式
			// */
			// contentType = "application/json";
			//
			// /**
			// * STEP5：组装请求参数
			// */
			// JSONObject jsonBody = new JSONObject();
			//
			// // 获取分页监控点资源参数
			//
			// jsonBody.put("pageNo", pageNo);
			// jsonBody.put("pageSize", pageSize);
			// jsonBody.put("indexCodes", "[" + indexCodes + "]");
			// body = jsonBody.toJSONString().replace("\"[", "[\"").replace("]\"", "\"]");

		}

		/**
		 * STEP6：调用接口
		 */
		String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);// post请求application/json类型参数
		return result;
	}

	//	设备信息推送
	public void sbxxPush() {
		try {
			Integer pageNo = 1;
			Integer pageSize = 1;
			Integer type = 2; // 获取分页信息监控点信息

			String result = GetCameraPreviewURL(pageNo, pageSize, type);

			JSONObject jsonString = JSONObject.parseObject(result);

			JSONObject jsonData = jsonString.getJSONObject("data");

			Integer jsonTotal = jsonData.getInteger("total"); // 总条数

			System.out.println("总条数为----------" + jsonTotal + "----------------------");

			Integer pageCount = (int) Math.ceil(jsonTotal / 1000f);
			System.err.println("总页数为----------" + pageCount + "----------------------");
			String resultData = null;
			
			while (pageNo <= pageCount) {
				System.out.println("页数为----------" + pageNo + "----------------------");
				resultData = GetCameraPreviewURL(pageNo, 1000, type);
				saveSbxx(resultData);
				pageNo++;
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void saveSbxx(String resultData) {
		// 将String转化为json数据
		JSONObject objData = JSONObject.parseObject(resultData);
		JSONObject jsonObjData = objData.getJSONObject("data");
		JSONArray arrayList = JSON.parseArray(jsonObjData.getString("list"));
		Iterator<Object> it = arrayList.iterator();
		while (it.hasNext()) {
			JSONObject obj = (JSONObject) it.next();
			HKSBXX hksbxx = JSONObject.parseObject(obj.toString(), HKSBXX.class);
			String cameraIndexCode = hksbxx.getCameraIndexCode();
			hksbxx.setID(cameraIndexCode);
			hksbxxMapper.delete(cameraIndexCode);
			hksbxxMapper.save(hksbxx);
		}
	}
	public void zzjgPush() {
		RootResult<Object> r = new RootResult<Object>();
		try {
			Integer pageNo = 1;
			Integer pageSize = 1;
			// 获取区域信息
			Integer type = 1;

			String result = GetCameraPreviewURL(pageNo, pageSize, type);

			JSONObject jsonString = JSONObject.parseObject(result);

			JSONObject jsonData = jsonString.getJSONObject("data");

			// 总条数
			Integer jsonTotal = jsonData.getInteger("total");

			System.out.println("总条数为----------" + jsonTotal + "----------------------");

			String resultData = GetCameraPreviewURL(1, 1000, type);
			// 将String转化为json数据
			JSONObject objData = JSONObject.parseObject(resultData);

			JSONObject jsonObjData = objData.getJSONObject("data");

			JSONArray arrayList = JSON.parseArray(jsonObjData.getString("list"));

			r.setData(arrayList);
			Iterator<Object> it = arrayList.iterator();
			while (it.hasNext()) {
				JSONObject obj = (JSONObject) it.next();
				HKZZJG hkzzjg = JSONObject.parseObject(obj.toString(), HKZZJG.class);
				String indexCode = hkzzjg.getIndexCode();
				hkzzjg.setID(indexCode);
				hkzzjgMapper.delete(indexCode);
				hkzzjgMapper.save(hkzzjg);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	//*视频摄像头********************************************************************************//
	
	
	//*数据抓取********************************************************************************//
    @Autowired
    private TableCommentMapper tableCommentMapper;
    @Autowired
    private GrabResultMapper grabResultMapper;
    
    
    public void grabAllScheduled() {
    	try {
    		//当前时间
    		String endTime = DateUtil.getCurrentDateStr();
    		//当前时间前一天
    		String startTime = DateUtil.getPreDateByDate(endTime,2);
    		List<TableCommentEntity> tableCommentMaps = tableCommentMapper.findAll();
    		for (TableCommentEntity tableCommentEntity : tableCommentMaps) {
    			String table_NAME = tableCommentEntity.getTABLE_NAME();
    	        //获取所有粮库的外网地址
    	        List<LiangKuApiEntity> list = liangKuApiMapper.findAllByLiangKuApi();
    	        for (LiangKuApiEntity api : list) {
    	            run(api, InstructionTypeEnum.dataInstructionOfEveryMonth.getCode(), startTime,
    	            		endTime,table_NAME,api.getDwdm());
    	        }
    			Thread.sleep(5000);
    			System.out.print("\n***************线程睡眠5秒！****************\n");
    		}
    	} catch (Exception e) {
    		log.error(e);
    	}
    }
    /**
     * @描述 线程
     * @param：[map, i]
     * @rerurn： void
     * @author：蒋青茂
     * @date 2019-3-19
     */
    private void run(LiangKuApiEntity api, int type, String startTime, String endTime,String tableName,String dwdm) {
        executorService.execute(new Runnable() {
            @Override
            public void run() {
    			String result = "";
    			String name = api.getName();
    			String url = api.getUrl();
                try {
                    //发送心跳指令
                    boolean heart = connect(url,dwdm);
                    if (heart) {
                        //下发数据指令
                        if (dateIssue(url, type, startTime, endTime,tableName,dwdm)) {
                        	result = name + ":数据指令处理成功" + " 接口:" + url;
                            log.info(name + ":数据指令处理成功" + " 接口:" + url);
                        } else {
                        	result = name + ":数据指令处理失败" + " 接口:" + url;
                            log.error(name + ":数据指令处理失败" + " 接口:" + url);
                        }
                    } else {
                        log.error(name + ":心跳指令发送失败" + " 接口:" + url);
                        result = name + ":心跳指令处理失败" + " 接口:" + url;
                    }
                } catch (Exception e) {
                    log.error("线程异常", e);
                }
                
                
                //写入抓取结果
                int save = grabResultMapper.save(OrderID.get32(), tableName, url, result);
                System.out.println(tableName + "抓取结果写入------------------------"+save);
            }
        });
    }
    
    /**
     * @描述 心跳指令
     * @param：[value] 粮库的接口地址
     * @rerurn： boolean
     * @author：蒋青茂
     * @date 2019-3-19
     */
    private boolean connect(String url,String dwdm) throws InterruptedException {
        //心跳指令请求体数据
        try {
			InstructionParasDTO dto = new InstructionParasDTO();
			dto.setOrderId(UUID.randomUUID().toString());
			dto.setType(InstructionTypeEnum.hearBeatInstruction.getCode().toString());
			dto.setDwdm(dwdm);
			//进行最大次数的三次连接
			for (int j = 0; j < 3; j++) {
			    if (liangKuRest.hearIssure(url, dto)) {
			        return true;
			    }
			    Thread.sleep(10000);
			}
			return false;
		} catch (Exception e) {
			return false;
		}
    }
    
    /**
     * @描述 数据下发指令
     * @param：[value, i]  粮库的接口地址 、需要上传数据类别
     * @rerurn： java.lang.String
     * @author：蒋青茂
     * @date 2019-3-19
     */
    public Boolean dateIssue(String url, int type, String startTime, String endTime,String tableName,String dwdm) {
        String content = tableName;
////        switch (type) {
////            case 10:
////                content = liangKuConfig.getTenMinute();
////                break;
////            case 30:
////                content = liangKuConfig.getHalfHour();
////                break;
////            case 100:
////                content = liangKuConfig.getEveryMonth();
////                break;
////            case 200:
////                content = liangKuConfig.getEveryQuarter();
////                break;
////            case 300:
////                content = liangKuConfig.getEveryYear();
////                break;
////            default:
////                log.error("没有匹配的接口数据");
////        }
        InstructionParasDTO dto = new InstructionParasDTO();
        dto.setOrderId(UUID.randomUUID().toString());
        dto.setType(InstructionTypeEnum.dataInstruction.getCode().toString());
        //指令内容
        dto.setData(content);
        //开始时间
        dto.setStartTime(startTime);
        //结束时间
        dto.setEndTime(endTime);
        //单位代码（统一社会信用代码）
		dto.setDwdm(dwdm);
		//操作标志 n:未上传 a:未上传和以上传的数据（包括操作标志为d的数据） c:未上传和已上传的数据的数据（不包括操作标志为d的数据）
		dto.setCzbz("n");
        //发送指令
        return liangKuRest.instructionIssure(url, dto);
    }
    
	//*数据抓取********************************************************************************//
}
