package com.unis.hlht.cjlksj.server.controller;

import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.unis.hlht.cjlksj.server.entity.RestCountEntity;
import com.unis.hlht.cjlksj.server.entity.TableCommentEntity;
import com.unis.hlht.cjlksj.server.enums.instruction.InstructionTypeEnum;
import com.unis.hlht.cjlksj.server.mapper.RestCountEntityMapper;
import com.unis.hlht.cjlksj.server.mapper.TableCommentMapper;
import com.unis.hlht.cjlksj.server.server.InstructionServer;
import com.unis.hlht.cjlksj.server.server.RestCountServer;
import com.unis.hlht.cjlksj.server.utils.DateUtil;
import com.unis.hlht.cjlksj.server.utils.ExcelUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

/**
 * 功能描述: 指令通道controller
 *
 * @Auther: 蒋青茂
 * @Date: 2019-3-18 10:24
 */

@Log4j2
@SuppressWarnings("all")
@RestController
@Api(value = "Instruction controller", description = "心跳指令接口控制层")
@RequestMapping("/instruction")
public class InstructionController {

    @Autowired
    private InstructionServer instructionServer;
    @Autowired
    private RestCountServer restCountServer;

    /**
     * @param type 抓取类型为1 则手动抓取历史数据             废弃  -- 已CompleteScheduleConfig统一管理
     */
//    @Scheduled(cron = "0 58 23 ? * *")
//    public void grabAllScheduled() {
//    	try {
//    		//当前时间
//    		String endTime = DateUtil.getCurrentDateStr();
//    		//当前时间前一天
//    		String startTime = DateUtil.getPreDateByDate(endTime,2);
//    		
//    		List<TableCommentEntity> tableCommentMaps = restCountEntityServer.getTableCommentMaps();
//    		for (TableCommentEntity tableCommentEntity : tableCommentMaps) {
//    			String table_NAME = tableCommentEntity.getTABLE_NAME();
//    			instructionServer.excute(InstructionTypeEnum.dataInstructionOfEveryMonth.getCode(),
//    					startTime, endTime, table_NAME);
//    			Thread.sleep(5000);
//    			System.out.print("\n***************线程睡眠5秒！****************\n");
//    		}
//    	} catch (Exception e) {
//    		log.error(e);
//    	}
//    }
    /**
     * 下发数据指令
     *
     * @param startTime eg:2023-01-01 00:00:00
     * @param kdmc      表 LIANGKU_API 中 NAME 字段值
     * @param tableName 表 TABLE_COMMENT_MAP 中 TABLE_NAME 字段值
     */
    @PostMapping(value = "/grabAll")
    public void grabAll(@RequestParam(value = "startTime", required = false) String startTime,
                        @RequestParam(value = "kdmc", required = false) String kdmc,
                        @RequestParam(value = "tableName", required = false) String tableName) {
        try {
            //当前时间
            String endTime = DateUtil.getCurrentDateStr();
            if (StringUtils.isNotEmpty(tableName)) {
                if (StringUtils.isNotEmpty(kdmc)) {
                    instructionServer.excuteKd(InstructionTypeEnum.dataInstructionOfEveryMonth.getCode(),
                            startTime, endTime, tableName, kdmc);
                    System.out.println(kdmc + "表"+tableName+"数据抓取完" + "%%%%%%%%%%");
                } else {
                    instructionServer.excute(InstructionTypeEnum.dataInstructionOfEveryMonth.getCode(),
                            startTime, endTime, tableName);
                    System.out.println("表" + tableName + "数据抓取完" + "%%%%%%%%%%");
                }
            } else {
                List<TableCommentEntity> tableCommentMaps = restCountServer.getTableCommentMapsAll();
                for (TableCommentEntity tableCommentEntity : tableCommentMaps) {
                    String table_NAME = tableCommentEntity.getTABLE_NAME();
                    if (StringUtils.isNotEmpty(kdmc)) {
                        instructionServer.excuteKd(InstructionTypeEnum.dataInstructionOfEveryMonth.getCode(),
                                startTime, endTime, table_NAME, kdmc);
                        System.out.println(kdmc + "数据抓取完成，抓取表总计" + tableCommentMaps.size() + "%%%%%%%%%% ");
                    } else {
                        instructionServer.excute(InstructionTypeEnum.dataInstructionOfEveryMonth.getCode(),
                                startTime, endTime, table_NAME);
                    }
                    Thread.sleep(5000);
                    System.out.print("\n***************线程睡眠5秒！****************\n");
                }
            }
        } catch (Exception e) {
            log.error(e);
        }
    }

    /**
     * 每十分钟发送一次数据指令
     */
//    @Scheduled(cron = "0 0/10 * * * ?")
    @PostMapping(value = "/test")
    public void instructionOfTenMinute(@RequestParam String tableName) {
        try {
            //当前时间往前推十分钟
//            String startTime = DateUtil.getPreTime(DateUtil.getNowDate(), "-10");
            //当前时间
//            String endTime = DateUtil.getDateTime();
            String startTime = "2015-01-01 00:00:00";
            String endTime = "2020-01-01 00:00:00";
            instructionServer.excute(InstructionTypeEnum.dataInstructionOfTenMinute.getCode(), startTime, endTime, tableName);
        } catch (Exception e) {
            log.error(e);
        }
    }

    //
//    /**
//     * 每月最后一天(如：2019/5/31 23:59:00)发送一次数据指令
//     */
////    @Scheduled(cron = "0 59 23 L * ?")
    @PostMapping(value = "/grabMonthByTName")
    public void instructionOfEveryMonth(@RequestParam String tableName) {
        try {
            //当前月第一天
//            String startTime = DateUtil.getTheFirstDayAndTimeOfCurrentMonth();
            //当前月最后一天
//            String endTime = DateUtil.getDateTime();
            String startTime = "2015-01-01 00:00:00";
            String endTime = "2021-01-01 00:00:00";
            instructionServer.excute(InstructionTypeEnum.dataInstructionOfEveryMonth.getCode(),
                    startTime, endTime, tableName);
        } catch (Exception e) {
            log.error(e);
        }
    }

    //    /**
//     * 每季度最后一天发送一次数据指令
//     * 3/6/9/12底各执行次 如2019-03-31 23:59:00
//     */
////    @Scheduled(cron = "0 59 23 L 3/3 ?")
//    @PostMapping(value = "/test2")
//    public void instructionOfEveryQuarter() {
//        try {
//            //当前月往前推三个月的第一天
//            String startTime = DateUtil.getTheFirstDayOfSpecifiedMonth(DateUtil.getMonth(DateUtil.getNowDate()) - 2);
//            //当前月最后一天
//            String endTime = DateUtil.getDateTime();
//            instructionServer.excute(InstructionTypeEnum.dataInstructionOfEveryQuarter.getCode(), null, null);
//        } catch (Exception e) {
//            log.error(e);
//        }
//    }
//
//    /**
//     * 每年最后一天发送一次数据指令
//     * 2019-12-31 23:59:00
//     */
////    @Scheduled(cron = "0 59 23 L 12 ? *")
//    @PostMapping(value = "/test3")
//    public void instructionOfEveryYear() {
//        try {
//            //当年第一天
//            String startTime = DateUtil.getTheFirstDayOfSpecifiedMonth(1);
//            //当年最后一天
//            String endTime = DateUtil.getDateTime();
//            instructionServer.excute(InstructionTypeEnum.dataInstructionOfEveryQuarter.getCode(), null, null);
//        } catch (Exception e) {
//            log.error(e);
//        }
//    }
//

    /**
     * 统计上传数据-统计表需在数据库定义
     *
     * @param request
     * @param response
     */
    @ApiOperation(value = "统计上传数据", httpMethod = "GET")
    @GetMapping(value = "/getSum")
    public void getSum(HttpServletRequest request, HttpServletResponse response) {
        try {
            //获取统计数据
            List<List<RestCountEntity>> count = restCountServer.getCount();
            //先删除
            restCountServer.deleteByTime();
            //数据存入
            restCountServer.saveCount(count);
            //最终数据统计
            List<Map> restSum = restCountServer.getRestSum();
            for (Map map : restSum) {
                String qymc = (String) map.get("企业名称");
                List<Map> restSumDetail = restCountServer.getRestSumDetail(qymc);
                map.put("明细", restSumDetail);
            }

            //excel文件名
            String fileName = "粮库上传明细" + System.currentTimeMillis() + ".xls";
            //excel标题
            String[] title = {"企业名称", "统一社会信用代码", "上传表张数", "上传数据总量", "明细"};
            //sheet名
            String sheetName = "粮库上传明细";
            String[][] content = new String[restSum.size()][5];
            for (int i = 0; i < restSum.size(); i++) {
                Map obj = restSum.get(i);
                content[i][0] = obj.get("企业名称").toString();
                content[i][1] = obj.get("统一社会信用代码").toString();
                content[i][2] = obj.get("上传表张数").toString();
                content[i][3] = obj.get("上传数据总量").toString();
                List<Map> detail = (List<Map>) obj.get("明细");
                StringBuffer stringBuffer = new StringBuffer();
                for (Map map : detail) {
                    stringBuffer.append(map.get("表名") + ":" + map.get("表数据量") + "条; ");
                }
                content[i][4] = stringBuffer.toString();
            }
            //创建HSSFWorkbook
            HSSFWorkbook wb = ExcelUtil.getHSSFWorkbook(sheetName, title, content, null);
            //响应到客户端
            try {
                this.setResponseHeader(response, fileName);
                OutputStream outputStream = response.getOutputStream();
                wb.write(outputStream);
                outputStream.flush();
                outputStream.close();
            } catch (Exception e) {

            }
        } catch (Exception e) {
            e.printStackTrace();
//            log.error(e);
        }
    }

    /**
     * 统计上传数据-统计表需在数据库定义
     *
     * @param request
     * @param response
     */
    @ApiOperation(value = "统计上传数据-所有表", httpMethod = "GET")
    @GetMapping(value = "/getAllSum")
    public void getAllSum(HttpServletRequest request, HttpServletResponse response) {
        try {
            //获取统计数据
            List<List<RestCountEntity>> count = restCountServer.getAllCount();
            //先删除
            restCountServer.deleteByTime();
            //数据存入
            restCountServer.saveCount(count);
            //最终数据统计
            List<Map> restSum = restCountServer.getRestSum();
            for (Map map : restSum) {
                String qymc = (String) map.get("企业名称");
                List<Map> restSumDetail = restCountServer.getRestSumDetail(qymc);
                map.put("明细", restSumDetail);
            }

            //excel文件名
            String currentDateTime16Str = DateUtil.getCurrentStringTimestamp();
            String fileName = "粮库上传明细" + currentDateTime16Str + ".xls";
            //excel标题
            String[] title = {"企业名称", "统一社会信用代码", "上传表张数", "上传数据总量", "明细"};
            //sheet名
            String sheetName = "粮库上传明细";
            String[][] content = new String[restSum.size()][5];
            for (int i = 0; i < restSum.size(); i++) {
                Map obj = restSum.get(i);
                content[i][0] = obj.get("企业名称").toString();
                content[i][1] = obj.get("统一社会信用代码").toString();
                content[i][2] = obj.get("上传表张数").toString();
                content[i][3] = obj.get("上传数据总量").toString();
                List<Map> detail = (List<Map>) obj.get("明细");
                StringBuffer stringBuffer = new StringBuffer();
                for (Map map : detail) {
                    stringBuffer.append(map.get("表名") + ":" + map.get("表数据量") + "条; ");
                }
                content[i][4] = stringBuffer.toString();
            }
            //创建HSSFWorkbook
            HSSFWorkbook wb = ExcelUtil.getHSSFWorkbook(sheetName, title, content, null);
            //响应到客户端
            try {
                this.setResponseHeader(response, fileName);
                OutputStream outputStream = response.getOutputStream();
                wb.write(outputStream);
                outputStream.flush();
                outputStream.close();
            } catch (Exception e) {

            }
        } catch (Exception e) {
            e.printStackTrace();
//            log.error(e);
        }
    }

    /**
     * 统计上传数据-统计表需在数据库定义
     *
     * @param request
     * @param response
     */
    @ApiOperation(value = "统计上传数据V2", httpMethod = "GET")
    @GetMapping(value = "/getSumV2")
    public void getSumV2(HttpServletRequest request, HttpServletResponse response) {
        try {
            //获取统计数据
            List<List<RestCountEntity>> count = restCountServer.getCountV2();
            //先删除
            restCountServer.deleteByTime();
            //数据存入
            restCountServer.saveCount(count);
            //最终数据统计
            List<Map> restSum = restCountServer.getRestSum();

            Collections.sort(restSum, new Comparator<Map>() {
                @Override
                public int compare(Map o1, Map o2) {
                    //升序
                    return o1.get("统一社会信用代码").toString().compareTo(o2.get("统一社会信用代码") + "");
                }
            });

            for (Map map : restSum) {
                String qymc = (String) map.get("企业名称");
                List<Map> restSumDetail = restCountServer.getRestSumDetail(qymc);
                map.put("明细", restSumDetail);
            }

            //excel文件名
            String fileName = "粮库上传明细" + System.currentTimeMillis() + ".xls";
            //excel标题
            String[] title = {"企业名称", "统一社会信用代码", "上传表张数", "上传数据总量", "明细"};
            //sheet名
            String sheetName = "粮库上传明细";
            String[][] content = new String[restSum.size()][5];
            for (int i = 0; i < restSum.size(); i++) {
                Map obj = restSum.get(i);
                content[i][0] = obj.get("企业名称").toString();
                content[i][1] = obj.get("统一社会信用代码").toString();
                content[i][2] = obj.get("上传表张数").toString();
                content[i][3] = obj.get("上传数据总量").toString();
                List<Map> detail = (List<Map>) obj.get("明细");
                StringBuffer stringBuffer = new StringBuffer();
                for (Map map : detail) {
                    stringBuffer.append(map.get("表名") + ":" + map.get("表数据量") + "条; ");
                }
                content[i][4] = stringBuffer.toString();
            }
            //创建HSSFWorkbook
            HSSFWorkbook wb = ExcelUtil.getHSSFWorkbook(sheetName, title, content, null);
            //响应到客户端
            try {
                this.setResponseHeader(response, fileName);
                OutputStream outputStream = response.getOutputStream();
                wb.write(outputStream);
                outputStream.flush();
                outputStream.close();
            } catch (Exception e) {

            }
        } catch (Exception e) {
            e.printStackTrace();
//            log.error(e);
        }
    }

    /**
     * 统计上传数据-统计表需在数据库定义
     *
     * @param request
     * @param response
     */
    @ApiOperation(value = "统计上传数据V2", httpMethod = "GET")
    @GetMapping(value = "/getSumAllV2")
    public void getSumAllV2(HttpServletRequest request, HttpServletResponse response) {
        try {
            //获取统计数据
            List<List<RestCountEntity>> count = restCountServer.getCountV2();
            //先删除
            restCountServer.deleteByTime();
            //数据存入
            restCountServer.saveCount(count);
            //最终数据统计
            List<Map> restSum = restCountServer.getRestSum();

            Collections.sort(restSum, new Comparator<Map>() {
                @Override
                public int compare(Map o1, Map o2) {
                    //升序
                    return o1.get("统一社会信用代码").toString().compareTo(o2.get("统一社会信用代码") + "");
                }
            });

            for (Map map : restSum) {
                String qymc = (String) map.get("企业名称");
                List<Map> restSumDetail = restCountServer.getRestSumDetail(qymc);
                map.put("明细", restSumDetail);
            }

            //excel文件名
            String fileName = "粮库上传明细" + System.currentTimeMillis() + ".xls";
            //excel标题
            String[] title = {"企业名称", "统一社会信用代码", "上传表张数", "上传数据总量", "明细"};
            //sheet名
            String sheetName = "粮库上传明细";
            String[][] content = new String[restSum.size()][5];
            for (int i = 0; i < restSum.size(); i++) {
                Map obj = restSum.get(i);
                content[i][0] = obj.get("企业名称").toString();
                content[i][1] = obj.get("统一社会信用代码").toString();
                content[i][2] = obj.get("上传表张数").toString();
                content[i][3] = obj.get("上传数据总量").toString();
                List<Map> detail = (List<Map>) obj.get("明细");
                StringBuffer stringBuffer = new StringBuffer();
                for (Map map : detail) {
                    stringBuffer.append(map.get("表名") + ":" + map.get("表数据量") + "条; ");
                }
                content[i][4] = stringBuffer.toString();
            }
            //创建HSSFWorkbook
            HSSFWorkbook wb = ExcelUtil.getHSSFWorkbook(sheetName, title, content, null);
            //响应到客户端
            try {
                this.setResponseHeader(response, fileName);
                OutputStream outputStream = response.getOutputStream();
                wb.write(outputStream);
                outputStream.flush();
                outputStream.close();
            } catch (Exception e) {

            }
        } catch (Exception e) {
            e.printStackTrace();
//            log.error(e);
        }
    }

    //发送响应流方法
    public void setResponseHeader(HttpServletResponse response, String fileName) {
        try {
            try {
                fileName = new String(fileName.getBytes(), "ISO8859-1");
            } catch (UnsupportedEncodingException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            response.setContentType("application/octet-stream;charset=utf-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            response.addHeader("Pargam", "no-cache");
            response.addHeader("Cache-Control", "no-cache");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    @Autowired
    private TableCommentMapper tableCommentMapper;
    @Autowired
    private RestCountEntityMapper restCountEntityMapper;

    /**
     * 删除某个单位的所有数据---定义在统计表中的数据
     *
     * @param tyshxydm
     * @param dwdm
     */
    @PostMapping(value = "/deleteByTyshxydmOrDwdm")
    public void deleteByTyshxydmOrDwdm(@RequestParam(required = false) String tyshxydm, @RequestParam(required = false) String dwdm) {
        if (StringUtils.isBlank(tyshxydm) && StringUtils.isBlank(dwdm)) {
            return;
        }
        List<TableCommentEntity> tableCommentEntities = tableCommentMapper.findAll();
        for (TableCommentEntity tableCommentEntity : tableCommentEntities) {
            restCountEntityMapper.deleteByTyshxydmOrDwdm(tableCommentEntity.getTABLE_NAME(), tyshxydm, dwdm);
        }
    }


    /**
     * 测试--用完删除
     *
     * @param type
     */
    @PostMapping(value = "/grabOneTbaleAndOneKd")
    public String grabTEST(
            @RequestParam(value = "table") String table,
            @RequestParam(value = "heart_kdmc") String heart_kdmc
    ) {
        try {
            //当前时间
            String endTime = DateUtil.getCurrentDateStr();
            //当前时间前一天
            String startTime = "2001-06-12 00:00:00";

            List<TableCommentEntity> tableCommentMaps = restCountServer.getTableCommentMapsAll();
            for (TableCommentEntity tableCommentEntity : tableCommentMaps) {
                String table_NAME = tableCommentEntity.getTABLE_NAME();
                if (table_NAME.equals(table)) {
                    System.out.println("-----------------------" + table);
                    instructionServer.excuteKd(InstructionTypeEnum.dataInstructionOfEveryMonth.getCode(),
                            startTime, endTime, table_NAME, heart_kdmc);
                }

            }
        } catch (Exception e) {
            log.error(e);
            return "指令下发fail";
        }
        return "指令下发success";
    }


    /**
     * 删除某个单位的所有数据---定义在统计表中的数据---用完删除
     *
     * @param tyshxydm
     * @param dwdm
     */
    @PostMapping(value = "/deleteByTyshxydmTmp")
    public void deleteByTyshxydmTmp(@RequestParam String tyshxydm,
                                    @RequestParam String table) {
        if (StringUtils.isBlank(tyshxydm) && StringUtils.isBlank(table)) {
            return;
        }
        int deleteByTyshxydm = restCountEntityMapper.deleteByTyshxydm(table, tyshxydm, null);
        System.out.println(deleteByTyshxydm);
    }
}