package com.unis.hlht.cjlksj.server.server;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.unis.hlht.cjlksj.server.common.check.CheckCommonData;
import com.unis.hlht.cjlksj.server.common.data.LiangKuConfig;
import com.unis.hlht.cjlksj.server.dto.upload.UploadDTO;
import com.unis.hlht.cjlksj.server.mapper.DataUploadMapper;
import com.unis.hlht.cjlksj.server.mapper.GeneralMapper;
import com.unis.hlht.cjlksj.server.mapper.SysUploadErrorResultMapper;
import com.unis.hlht.cjlksj.server.redisstream.StreamRedisQueue;
import com.unis.hlht.cjlksj.server.utils.LogicKeyUtil;
import com.unis.hlht.cjlksj.server.utils.MapUtils;
import com.unis.hlht.cjlksj.server.utils.ResponseWrapper;
import com.unis.hlht.cjlksj.server.utils.SecureMngr;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.InputStream;
import java.util.*;

@Slf4j
@Service
@SuppressWarnings("all")
public class DataUpLoadServer {

    private static final String CZBZ_D = "d";
    private static final String COLUMN_ID = "id";
    private static final String COLUMN_CZBZ = "czbz";

    @Autowired
    private DataUploadMapper dataUploadMapper;

    @Autowired
    private LiangKuConfig liangKuConfig;


    @Autowired
    private GeneralMapper generalMapper;

    @Autowired
    private SysUploadErrorResultMapper sysUploadErrorResultMapper;

    @Autowired
    private DataCurdServer dataCurdServer;

    @Autowired
    private StreamRedisQueue streamRedisQueue;

    /**
     * 数据上传
     *
     * @param dto
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public ResponseWrapper uploadData(UploadDTO dto) throws Exception {
        //主体数据
        String encryptionBody = String.valueOf(dto.getBody());
        log.info("接收数据:" + encryptionBody);
        //主体数据解密
        String bodyStr = SecureMngr.decryptByAES(SecureMngr.AES_KEY, encryptionBody);
//        String bodyStr = JSONObject.toJSONString(dto.getBody());
        //处理数据条数初始化
        int countDataNum = 0;
        //加载数据校验规则
        Properties properties = new Properties();
        properties.load(DataUpLoadServer.class.getClassLoader().getResourceAsStream("config/config.yml"));
        //是否需要数据校验
        boolean needCheck = Boolean.valueOf(properties.getProperty("needCheckXml"));
        //校验结果
        boolean checkRes = true;
        String msg = "";
        if (StringUtils.isNotBlank(bodyStr)) {
            if (needCheck) {
                org.dom4j.Document document = getDom(DataUpLoadServer.class.getClassLoader()
                        .getResourceAsStream("config/shanxi.xml"));
                Element root = document.getRootElement();
                List<Node> columnList = null;
                Element tableEle = null;
                Element columnEle = null;
                int columnSize = 0;
                JSONArray jsonArray = JSONArray.parseArray(bodyStr);
                JSONObject jsonObject = null;
                Set<Map.Entry<String, Object>> set = null;
                String tablename = null, columnname = null, value = null, regex = null, def = null, remark = null, judgmentBasis = null;
                boolean required = false;
                JSONArray data = null;
                JSONObject dataItem = null;
                //报错返回
                point:
                for (int i = 0, size = jsonArray.size(); i < size; i++) {
                    jsonObject = jsonArray.getJSONObject(i);
                    set = jsonObject.entrySet();
                    for (Map.Entry<String, Object> entry : set) {
                        // 表名
                        tablename = entry.getKey().toUpperCase().trim();
                        tableEle = (Element) root.selectSingleNode("/tables/table[@name='" + tablename + "']");
                        if (tableEle != null) {
                            columnList = tableEle.selectNodes("column[@regex|@default|@required]");
                            columnSize = columnList != null ? columnList.size() : 0;
                            if (columnSize > 0) {
                                // 粮库上传数据集合
                                data = (JSONArray) entry.getValue();
                                for (int j = 0, dataSize = data.size(); j < dataSize; j++) {
                                    // 粮库上传数据集合中单个数据元素
                                    dataItem = data.getJSONObject(j);
                                    // 临时：操作标志为d的不在进行数据校验
                                    String czbz = dataItem.getString("CZBZ");
                                    if (StringUtils.isBlank(czbz)) {
                                        czbz = dataItem.getString("czbz");
                                    }
                                    if ("d".equalsIgnoreCase(czbz)) {
                                        continue;
                                    }
                                    // 成品粮库存和原粮库存校验分别取了各自的数据校验规则
                                    if ("ODS_UP_LSKCXXJK".equalsIgnoreCase(tablename)) {
                                        String lspzdm = StringUtils.isNotBlank(dataItem.getString("LSPZDM")) ?
                                                dataItem.getString("LSPZDM") : dataItem.getString("lspzdm");
                                        if (CheckCommonData.CPL_LSPZDMS.contains(lspzdm)) {
                                            tableEle = (Element) root.selectSingleNode("/tables/table[@name='" + tablename + "_CPL" + "']");
                                            columnList = tableEle.selectNodes("column[@regex|@default|@required]");
                                            columnSize = columnList != null ? columnList.size() : 0;
                                        }
                                    }
                                    for (int x = 0; x < columnSize; x++) {
                                        columnEle = (Element) columnList.get(x);
                                        columnname = columnEle.getTextTrim().toUpperCase();
                                        regex = columnEle.attributeValue("regex");
                                        def = columnEle.attributeValue("default");
                                        remark = columnEle.attributeValue("remark");
                                        judgmentBasis = columnEle.attributeValue("judgmentBasis");
                                        required = Boolean.valueOf(columnEle.attributeValue("required"));
                                        if (dataItem.containsKey(columnname) || (dataItem.containsKey((columnname = columnname.toLowerCase())))) {
                                            value = dataItem.getString(columnname);
                                            if (value != null && value.length() > 0) {
                                                if (regex != null) {
                                                    checkRes = value.trim().matches(regex);
                                                    if (!checkRes) {
                                                        msg = "表<" + tablename + ">,第<" + j + "/" + dataSize + ">列,字段<" + columnname + ">,值<"
                                                                + value + ">不匹配近似模糊规则<" + regex + ">,详细规则<" + remark + ">";
                                                        break point;
                                                    }
                                                }
                                            } else {
                                                if (StringUtils.isNotBlank(judgmentBasis) &&
                                                        isNeedUpload(tablename, judgmentBasis, columnname, dataItem.getString(judgmentBasis))) {
                                                    // 根据判断依据列，判断当前列是否需要上传数值，进入此代码块为需要上传但未上传：记录错误信息并标记未通过数据校验
                                                    msg = "表<" + tablename + ">,第<" + j + "/" + dataSize + ">列,字段<" + columnname
                                                            + ">,值为空.不匹配近似模糊规则<" + regex + ">,详细规则<" + remark + ">";
                                                    checkRes = false;
                                                    break point;
                                                }
                                                if (def != null && !required) {
                                                    dataItem.put(columnname, def);
                                                } else if (required) {
                                                    msg = "表<" + tablename + ">,第<" + j + "/" + dataSize + ">列,字段<" + columnname
                                                            + ">,值为空.不匹配近似模糊规则<" + regex + ">,详细规则<" + remark + ">";
                                                    checkRes = false;
                                                    break point;
                                                }
                                            }
                                        } else {
                                            // 进入此判断：上传的数据中缺少我们要检查的字段。
                                            if (StringUtils.isNotBlank(judgmentBasis) &&
                                                    isNeedUpload(tablename, judgmentBasis, columnname, dataItem.getString(judgmentBasis))) {
                                                // 根据判断依据列，判断当前列是否需要上传数值，进入此代码块为需要上传但未上传：记录错误信息并标记未通过数据校验
                                                msg = "表<" + tablename + ">,第<" + j + "/" + dataSize + ">列,字段<" + columnname
                                                        + ">,值为空.不匹配近似模糊规则<" + regex + ">,详细规则<" + remark + ">";
                                                checkRes = false;
                                                break point;
                                            }
                                            if (def != null && !required) {
                                                dataItem.put(columnname, def);
                                            } else if (required) {
                                                msg = "表<" + tablename + ">,第<" + j + "/" + dataSize + ">列,字段<" + columnname
                                                        + ">缺失,近似模糊规则<" + regex + ">,详细规则<" + remark + ">";
                                                checkRes = false;
                                                break point;
                                            }
                                        }
                                    }
                                }
                            }
                        } else {
                            msg = "表<" + tablename + ">不存在";
                            checkRes = false;
                            break;
                        }
                    }
                }
                bodyStr = JSONArray.toJSONString(jsonArray);
                log.error(msg);
            }
            //判断是否通过校验，通过后需处理数据
            if (checkRes) {
                String body = bodyStr.substring(0, bodyStr.lastIndexOf("]") - 1);
                if (body.length() > 1) {
                    //表名
                    String tableName = body.substring(body.indexOf("{") + 2, body.indexOf(":") - 1);
                    //表数据
                    String tableDatasStr = body.substring(body.indexOf(":") + 1);
                    List<Object> tableDatas = JSONArray.parseArray(tableDatasStr);
                    System.out.println("------表：" + tableName + " ------数据上传" + tableDatas.size() + "条------");
                    //逻辑主键
                    List<String> keys = LogicKeyUtil.getprimeryKeys(tableName);
                    if (LogicKeyUtil.notVerify(tableName) == false) {
                        if (keys == null || keys.size() <= 0) {
                            log.error("未获取到表'" + tableName + "'的逻辑主键");
//                            continue;
                        }
                    }
                    //根据操作标志分为新增更新组和删除组
                    ArrayList<Map<String, Object>> iOrUArr = new ArrayList<>();
                    ArrayList<Map<String, Object>> delArr = new ArrayList<>();
                    //进行数据分离
                    for (Object tableData : tableDatas) {
                        //表数据(单条)
                        Map<String, Object> tableDataMap = (Map) tableData;
                        if ("ODS_UP_KQXXJK".equalsIgnoreCase(tableName)) {
                            String kqdmTemp = MapUtils.getObjectIgnoreCase(tableDataMap, "kqdm").toString();
                            String xzqhdm = MapUtils.getObjectIgnoreCase(tableDataMap, "xzqhdm").toString();
                            // 根据库区代码获取“展示用行政区划代码”（例如要把某个库区放到另一个市下进行统计）
                            // 原代码在原始xzqhdm字段上做修改
                            // 导致数据上传至国家局以后行政区划代码有错（展示里在市级层面多加了“省直属”，官方是没有这个行政区划的）
                            String zsyXzqhdm = getZsyKqXzqhdm(kqdmTemp);
                            tableDataMap.put("ZSYXZQHDM", StringUtils.isBlank(zsyXzqhdm) ? xzqhdm : zsyXzqhdm);
                        }
                        Object object = MapUtils.getObjectIgnoreCase(tableDataMap, COLUMN_CZBZ);
                        if (CZBZ_D.equalsIgnoreCase(object.toString())) {
                            delArr.add(tableDataMap);
                        } else {
                            iOrUArr.add(tableDataMap);
                        }
                    }
                    //先对删除组进行处理
                    for (Map<String, Object> delDataMap : delArr) {
                        //根据逻辑主键查询数据库是否有此数据
                        if (LogicKeyUtil.notVerify(tableName) == false) {
                            List<Map> deletDatas = generalMapper.getByLogickey(tableName, keys, delDataMap);
                            for (Map deletData : deletDatas) {
                                Object idObj = MapUtils.getObjectIgnoreCase(deletData, COLUMN_ID);
                                String id = deletData != null ? (idObj == null ? null : String.valueOf(idObj)) : null;
                                String czbz = deletData != null ? (deletData.get(COLUMN_CZBZ) == null
                                        ? null : deletData.get(COLUMN_CZBZ).toString()) : null;
                                //数据库中无此条数据或有数据操作标志为d直接跳过，其他做更新操作标志为d
                                if (StringUtils.isNotBlank(czbz) && !CZBZ_D.equalsIgnoreCase(czbz)) {
                                    generalMapper.deleteByPrimaryK(id, tableName);
                                    // TODO 删除后发送指令通知国家局服务上传数据
                                    this.sendUploadMsgToGjjServer(id, tableName, dto.getDwdm());
                                }
                            }
                        }
                    }
                    //对新增、更新组进行处理
                    for (Map<String, Object> iOrUDataMap : iOrUArr) {
                        if (LogicKeyUtil.notVerify(tableName) == false) {
                            //根据逻辑主键查询数据库是否有此数据
                            List<Map> iOrUOrDdatas = generalMapper.getByLogickey(tableName, keys, iOrUDataMap);
                            //对新增和更新而言，做查询操作时，如果数据库中不存在此条数据或有数据且操作标志为d，则对上传数据的操作标志统一修改值为i，并记录，之后做批量插入。如果有数据且操作标志为i或者u，则对此条数据的操作标志统一修改为u，之后做更新操作。
                            ArrayList<Map> iOrUdatas = new ArrayList<>();
                            for (Map iOrUdata : iOrUOrDdatas) {
                                Object czbzObj = MapUtils.getObjectIgnoreCase(iOrUdata, COLUMN_CZBZ);
                                String czbz = iOrUdata != null ? (czbzObj == null ? null : czbzObj.toString()) : null;
                                if (StringUtils.isNotBlank(czbz) && !CZBZ_D.equalsIgnoreCase(czbz)) {
                                    // 需要更新的数据
                                    iOrUdatas.add(iOrUdata);
                                }
                            }
                            if (iOrUdatas != null && iOrUdatas.size() > 0) {
                                for (Map iOrUdata : iOrUdatas) {
                                    Object idObj = MapUtils.getObjectIgnoreCase(iOrUdata, COLUMN_ID);
                                    String id = iOrUdata != null ? (idObj == null ? null : String.valueOf(idObj)) : null;
                                    iOrUDataMap.put("id", id);
                                    iOrUDataMap.put("tableName", tableName);
                                    if ("ODS_UP_WJXXJK".equalsIgnoreCase(tableName)
                                            || "ODS_UP_WSDJCXXJK".equalsIgnoreCase(tableName)
                                            || "ODS_UP_CNSPTXXXJK".equalsIgnoreCase(tableName)
                                            || "ODS_UP_SPJKYCSJGJJBXXJK".equalsIgnoreCase(tableName)) {
                                        dataCurdServer.updateByPrimaryK(iOrUDataMap, tableName);
                                    } else {
                                        generalMapper.updateByPrimaryK(iOrUDataMap);
                                    }
                                    // TODO 更新后发送指令通知国家局服务上传数据
                                    this.sendUploadMsgToGjjServer(id, tableName, dto.getDwdm());
                                }
                            } else {
                                iOrUDataMap.put("tableName", tableName);
                                if ("ODS_UP_WJXXJK".equalsIgnoreCase(tableName)
                                        || "ODS_UP_WSDJCXXJK".equalsIgnoreCase(tableName)
                                        || "ODS_UP_CNSPTXXXJK".equalsIgnoreCase(tableName)
                                        || "ODS_UP_SPJKYCSJGJJBXXJK".equalsIgnoreCase(tableName)) {
                                    dataCurdServer.insertData(iOrUDataMap, tableName);
                                } else {
                                    generalMapper.insertData(iOrUDataMap);
                                }
                                // TODO 新增后发送指令通知国家局服务上传数据
                                this.sendUploadMsgToGjjServer(null, tableName, dto.getDwdm());
                            }
                        } else {
                            iOrUDataMap.put("tableName", tableName);
                            if ("ODS_UP_WJXXJK".equalsIgnoreCase(tableName)
                                    || "ODS_UP_WSDJCXXJK".equalsIgnoreCase(tableName)
                                    || "ODS_UP_CNSPTXXXJK".equalsIgnoreCase(tableName)
                                    || "ODS_UP_SPJKYCSJGJJBXXJK".equalsIgnoreCase(tableName)) {
                                dataCurdServer.insertData(iOrUDataMap, tableName);
                            } else {
                                generalMapper.insertData(iOrUDataMap);
                            }
                            // TODO 新增后发送指令通知国家局服务上传数据
                            this.sendUploadMsgToGjjServer(null, tableName, dto.getDwdm());
                        }
                    }
                    countDataNum += tableDatas.size();
                }
                return ResponseWrapper.success(dto, String.valueOf(countDataNum));
            } else {
                //持久化错误信息到数据库
//                sysUploadErrorResultMapper.save(dto.getId(), dto.getOrgNo(), dto.getOrgName(), msg);
                return ResponseWrapper.error(dto, msg);
            }
        } else {
            msg = "正文内容为空";
            return ResponseWrapper.error(dto, msg);
        }
    }

    /**
     * 粮库上传数据条数计算
     */
    private int countBodyDatasNum(String body) {
        String tableDatasStr = body.substring(body.indexOf(":") + 1);
        List<Object> tableDatas = JSONArray.parseArray(tableDatasStr);
        return tableDatas.size();
    }

    public static org.dom4j.Document getDom(InputStream in) {
        org.dom4j.Document document = null;
        SAXReader saxReader = null;
        saxReader = new SAXReader();
        if (in != null) {
            try {
                document = saxReader.read(in);
            } catch (DocumentException e) {
                e.printStackTrace();
            }
        }
        return document;
    }

    /**
     * 根据判断依据列，判断当前检查列是否需要上传数值
     *
     * @param tableName          表名
     * @param judgmentBasie      判断依据列名
     * @param columnName         当前检查列名
     * @param judgmentBasieValue 当前上传数据中判断依据列的值
     * @return
     */
    private boolean isNeedUpload(String tableName, String judgmentBasie, String columnName, String judgmentBasieValue) {
        if ("ODS_UP_LSKCXXJK".equalsIgnoreCase(tableName)) {
            return lskcxxJudge(judgmentBasie, columnName, judgmentBasieValue);
        } else if ("ODS_UP_LSRKXXJK".equalsIgnoreCase(tableName)) {
            return lsrkxxJudge(judgmentBasie, columnName, judgmentBasieValue);
        } else if ("ODS_UP_LSRKJSXXJK".equalsIgnoreCase(tableName) || "ODS_UP_LSCKJSXXJK".equalsIgnoreCase(tableName)) {
            return lscrkjsxxJudge(judgmentBasie, columnName, judgmentBasieValue);
        }
        return false;
    }

    // 库存信息
    private boolean lskcxxJudge(String judgmentBasie, String columnName, String judgmentBasieValue) {
        if ("HWZT".equalsIgnoreCase(judgmentBasie)) {
            // 货位（油罐）状态 1:空仓 2:入库中 3:封仓 4:出库中 9:其他
            // 默认货位状态为2，3，4时，当前检查列是必须上传的
            if ("FCRQ".equalsIgnoreCase(columnName)) {
                // 封仓日期:yyyy-MM-dd（货位（油罐）状态为3、4时必填）
                if ("3".equalsIgnoreCase(judgmentBasieValue) || "4".equalsIgnoreCase(judgmentBasieValue)) {
                    return true;
                }
            } else if ("CCWCSJ".equalsIgnoreCase(columnName)) {
                // 出仓完成时间:最后一车粮食出仓时间。格式：yyyy-MM-dd HH:mm:ss（货位（油罐）状态为1时必填）
                if ("1".equalsIgnoreCase(judgmentBasieValue)) {
                    return true;
                }
            } else {
                if ("2".equalsIgnoreCase(judgmentBasieValue) || "3".equalsIgnoreCase(judgmentBasieValue) || "4".equalsIgnoreCase(judgmentBasieValue)) {
                    return true;
                }
            }
        }
        return false;
    }

    // 粮食入库信息
    private boolean lsrkxxJudge(String judgmentBasie, String columnName, String judgmentBasieValue) {
        if ("KLYY".equalsIgnoreCase(columnName)) {
            // 扣量原因:其他扣量不为0时，需注明原因
            if (StringUtils.isNotBlank(judgmentBasieValue) && Integer.valueOf(judgmentBasieValue) != 0) {
                return true;
            }
        } else if ("ZKHYY".equalsIgnoreCase(columnName)) {
            // 增扣价原因：增扣价不为0时注明原因
            if (StringUtils.isNotBlank(judgmentBasieValue) && Integer.valueOf(judgmentBasieValue) != 0) {
                return true;
            }
        }
        return false;
    }

    // 粮食出、入库结算信息
    private boolean lscrkjsxxJudge(String judgmentBasie, String columnName, String judgmentBasieValue) {
        // 结算方式： 0：现金 1：转账
        if ("JSFS".equalsIgnoreCase(judgmentBasie)) {
            // 结算方式为1转账时，默认当前检查列为必填
            if ("1".equalsIgnoreCase(judgmentBasieValue)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断库区的行政区划代码是否需要处理，是的话需要手动更新行政区划代码
     *
     * @param kqdm
     * @return
     */
    private String getZsyKqXzqhdm(String kqdm) {
        if (CheckCommonData.SZS_KQDMS.contains(kqdm)) {
            // 省直属
            return "612001";
        }
        if (CheckCommonData.XIAN_XXXQ_KQDMS.contains(kqdm)) {
            // 西安市-西咸新区
            return "610131";
        }
        if (CheckCommonData.XIAN_LHQ_KQDMS.contains(kqdm)) {
            // 西安市-莲湖区
            return "610104";
        }
        if (CheckCommonData.XIAN_LHQ_KQDMS.contains(kqdm)) {
            // 西安市-雁塔区
            return "610113";
        }

        return null;
    }

    /**
     * 发送
     *
     * @param busId     数据id 可为空
     * @param tableName 表名 不可为空
     * @param dwdm      单位统一社会信息代码 可为空
     */
    private void sendUploadMsgToGjjServer(String busId, String tableName, String dwdm) {
        // 通知要上传哪张表的哪条数据
        Map<String, String> value = new HashMap<>();
        value.put("busId", busId);
        value.put("tableName", tableName.trim());
        value.put("dwdm", StringUtils.isBlank(dwdm) ? null : dwdm.trim());
        streamRedisQueue.produce(value);
    }


}
