package com.unis.hlht.cjlksj.server.server;

import com.unis.hlht.cjlksj.server.common.data.LiangKuConfig;
import com.unis.hlht.cjlksj.server.common.rest.LiangKuRest;
import com.unis.hlht.cjlksj.server.dto.instruction.InstructionParasDTO;
import com.unis.hlht.cjlksj.server.entity.LiangKuApiEntity;
import com.unis.hlht.cjlksj.server.enums.instruction.InstructionTypeEnum;
import com.unis.hlht.cjlksj.server.mapper.GrabResultMapper;
import com.unis.hlht.cjlksj.server.mapper.LiangKuApiMapper;
import com.unis.hlht.cjlksj.server.utils.DateUtil;
import com.unis.hlht.cjlksj.server.utils.OrderID;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.UUID;
import java.util.concurrent.ExecutorService;

/**
 * 功能描述:
 *
 * @Auther: 蒋青茂
 * @Date: 2019-3-18 12:08
 */

@SuppressWarnings("all")
@Log4j2
@Service
public class InstructionServer {

    @Autowired
    private LiangKuRest liangKuRest;

    @Autowired
    private LiangKuApiMapper liangKuApiMapper;
   
    @Autowired
    private GrabResultMapper grabResultMapper;

    @Autowired
    private ExecutorService executorService;

    @Autowired
    private LiangKuConfig liangKuConfig;

    /**
     * @描述 定时任务执行下发指令
     * @param：[i]
     * @rerurn： void
     * @author：蒋青茂
     * @date 2019-3-19
     */
    public void excute(int type, String startTime, String endTime,String tableName) {
        //获取所有粮库的外网地址
        List<LiangKuApiEntity> list = liangKuApiMapper.findAllByLiangKuApi();
        for (LiangKuApiEntity api : list) {
            run(api, type, startTime, endTime,tableName);
        }
    }
    
    /**
     * @描述 定时任务执行下发指令----指定库
	 *
	 * @param startTime yyyy-MM-dd HH:mm:ss
     */
    public void excuteKd(int type, String startTime, String endTime,String tableName,String kdmc) {
    	//获取所有粮库的外网地址
    	List<LiangKuApiEntity> list = liangKuApiMapper.findAllByLiangKuApi();
    	for (LiangKuApiEntity api : list) {
    		String name = api.getName();
			if(name.equals(kdmc)) {
    			run(api, type, startTime, endTime,tableName);
    		}
    	}
    }


    /**
     * 心跳检查
     */
	public void excuteHeartCheck() {
		// 获取所有粮库的外网地址
		List<LiangKuApiEntity> list = liangKuApiMapper.findAll2(null);
		for (LiangKuApiEntity api : list) {
			
//			executorService.execute(new Runnable() {
//				@Override
//				public void run() {
					String name = api.getName();
					String result = "";
					String url = api.getUrl();
					String dwdm = api.getDwdm();
					// TODO Auto-generated method stub
					boolean heart = false;
					try {
						heart = connectCheck(url, dwdm);
					} catch (Exception e) {
						result = name + ":"+dwdm+"心跳检查_fail_exception_____" + " 接口:" + url;
						heart = false;
					}
					if (heart) {
						result = name + ":"+dwdm+"心跳检查_success__" + " 接口:" + url;
						liangKuApiMapper.updateIsOnlineById("1", DateUtil.getCurrentDateStr(), api.getId(), "T");
					} else {
						result = name + ":"+dwdm+"心跳检查_fail_____" + " 接口:" + url;
						liangKuApiMapper.updateIsOnlineById("0", DateUtil.getCurrentDateStr(), api.getId(),"F");
					}
					log.info(result);
//				}
//			});
		}
    }
    
	
	/**
	 * 心跳检查
	 * @return 
	 */
	public boolean excuteSimpleHeartCheck(String url,String dwdm) {
		// 获取所有粮库的外网地址
		try {
			return connectCheck(url,dwdm);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}
	
    /**
     * @描述 线程
     * @param：[map, i]
     * @rerurn： void
     * @author：蒋青茂
     * @date 2019-3-19
     */
    private void run(LiangKuApiEntity api, int type, String startTime, String endTime,String tableName) {
//        executorService.execute(new Runnable() {
//            @Override
//            public void run() {
    			String result = "";
    			String id = api.getId();
    			String name = api.getName();
    			String url = api.getUrl();
    			String dwdm = api.getDwdm();
                try {
                    //发送心跳指令
                    boolean heart = connect(url,dwdm);
                    if (heart) {
                        //下发数据指令
                        if (dateIssue(url, type, startTime, endTime,tableName,dwdm)) {
                        	result = name + ":数据指令处理成功" + " 接口:" + url;
                            log.info(name + ":数据指令处理成功" + " 接口:" + url);
                        } else {
                        	result = name + ":数据指令处理失败" + " 接口:" + url;
                            log.error(name + ":数据指令处理失败" + " 接口:" + url);
                        }
                    } else {
                        log.error(name + ":心跳指令发送失败" + " 接口:" + url);
                        result = name + ":心跳指令处理失败" + " 接口:" + url;
                    }
                } catch (Exception e) {
                    log.error("线程异常", e);
                }
                //写入抓取结果
                int save = grabResultMapper.save(OrderID.get32(), tableName, url, result);
                System.out.println(tableName + "抓取结果日志写入------------------------");
//            }
//        });
    }

    /**
     * @描述 心跳指令
     * @param：[value] 粮库的接口地址
     * @rerurn： boolean
     * @author：蒋青茂
     * @date 2019-3-19
     */
    private boolean connect(String url, String dwdm) throws InterruptedException {
        //心跳指令请求体数据
        try {
			InstructionParasDTO dto = new InstructionParasDTO();
			dto.setOrderId(UUID.randomUUID().toString());
			dto.setType(InstructionTypeEnum.hearBeatInstruction.getCode().toString());
			dto.setDwdm(dwdm);
			//进行最大次数的三次连接
			for (int j = 0; j < 3; j++) {
			    if (liangKuRest.hearIssure(url, dto)) {
			        return true;
			    }
//			    Thread.sleep(10000);
			}
			return false;
		} catch (Exception e) {
			return false;
		}
    }
    
    /**
     * @描述 心跳指令检测----单个地址测试
     * @param：[value] 粮库的接口地址
     * @rerurn： boolean
     * @author：蒋青茂
     * @date 2019-3-19
     */
    private boolean connectCheck(String url, String dwdm) throws InterruptedException {
    	//心跳指令请求体数据
    	try {
    		InstructionParasDTO dto = new InstructionParasDTO();
    		dto.setOrderId(UUID.randomUUID().toString());
    		dto.setType(InstructionTypeEnum.hearBeatInstruction.getCode().toString());
    		dto.setDwdm(dwdm);
    		//一次尝试
			if (liangKuRest.hearIssure(url, dto)) {
				return true;
			}
    		return false;
    	} catch (Exception e) {
    		return false;
    	}
    }

    /**
     * @描述 数据下发指令
     * @param：[value, i]  粮库的接口地址 、需要上传数据类别
     * @rerurn： java.lang.String
     * @author：蒋青茂
     * @date 2019-3-19
     */
    public Boolean dateIssue(String url, int type, String startTime, String endTime,String tableName,String dwdm) {
        String content = tableName;
////        switch (type) {
////            case 10:
////                content = liangKuConfig.getTenMinute();
////                break;
////            case 30:
////                content = liangKuConfig.getHalfHour();
////                break;
////            case 100:
////                content = liangKuConfig.getEveryMonth();
////                break;
////            case 200:
////                content = liangKuConfig.getEveryQuarter();
////                break;
////            case 300:
////                content = liangKuConfig.getEveryYear();
////                break;
////            default:
////                log.error("没有匹配的接口数据");
////        }
        InstructionParasDTO dto = new InstructionParasDTO();
        dto.setOrderId(UUID.randomUUID().toString());
        dto.setType(InstructionTypeEnum.dataInstruction.getCode().toString());
        //指令内容
        dto.setData(content);
        //开始时间
        dto.setStartTime(startTime);
        //结束时间
        dto.setEndTime(endTime);
		//单位代码（统一社会信用代码）
		dto.setDwdm(dwdm);
		//操作标志 n:未上传 a:未上传和以上传的数据（包括操作标志为d的数据） c:未上传和已上传的数据的数据（不包括操作标志为d的数据）
		dto.setCzbz("c");
        //发送指令
        return liangKuRest.instructionIssure(url, dto);
    }

}