package com.unis.hlht.cjlksj.server.utils;

import java.util.HashMap;
import java.util.Map;

import org.apache.poi.hssf.usermodel.*;

/**
 * @description:
 * @author:马伟栋
 * @version:v1.0
 * @create:2019/06/26-12:02
 **/
public class ExcelUtil {

    /**
     * 导出Excel
     *
     * @param sheetName sheet名称
     * @param title     标题
     * @param values    内容
     * @param wb        HSSFWorkbook对象
     * @return
     */
    public static HSSFWorkbook getHSSFWorkbook(String sheetName, String[] title, String[][] values, HSSFWorkbook wb) {

        // 第一步，创建一个HSSFWorkbook，对应一个Excel文件
        if (wb == null) {
            wb = new HSSFWorkbook();
        }

        // 第二步，在workbook中添加一个sheet,对应Excel文件中的sheet
        HSSFSheet sheet = wb.createSheet(sheetName);

        // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制
        HSSFRow row = sheet.createRow(0);

        // 第四步，创建单元格，并设置值表头 设置表头居中
        HSSFCellStyle style = wb.createCellStyle();
        // 创建一个居中格式
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);

        //声明列对象
        HSSFCell cell = null;
        //存储最大列宽  
        Map<Integer,Integer> maxWidth = new HashMap<Integer,Integer>(); 

        //创建标题
        for (int i = 0; i < title.length; i++) {
            cell = row.createCell(i);
            cell.setCellValue(title[i]);
            cell.setCellStyle(style);
            
            maxWidth.put(i,title[i].length());
        }
        
        //创建内容
        for (int i = 0; i < values.length; i++) {
            row = sheet.createRow(i + 1);
            for (int j = 0; j < values[i].length; j++) {
                //将内容按顺序赋给对应的列对象
                HSSFCell createCell = row.createCell(j);
				createCell.setCellValue(values[i][j]);

                int length = createCell.getStringCellValue().getBytes().length  * 256 + 200;  
                //这里把宽度最大限制到15000  
                if (length>15000){  
                	length = 15000;  
                }  
                maxWidth.put(j,Math.max(length,maxWidth.get(j)));          
            }
        }
        
        
//        // 列宽自适应  
        for (int i = 0; i < title.length; i++) {  
            sheet.setColumnWidth(i,maxWidth.get(i));  
        }  
        return wb;
    }
}

